# icon bar configuration
global xfIconBar
# icon bar: edit
set xfIconBar(bar,edit) {{conf {XFProcConfParametersDefault}} {pack {XFProcConfPacking}} {place XFProcConfPlacing} {bind {XFProcConfBinding}} {widgets {XFProcIconBarShow widgets}} {Iconbar-separator { }} {procs XFProcProgProcs} {global XFProcProgGlobals} {tree XFProcProgWidgetTree} {Iconbar-separator { }} {cut XFProcEditCut} {copy XFProcEditCopy} {paste XFProcEditPaste} {delete XFProcEditDelete} {Iconbar-separator { }} {question {XFProcHelpHelp XF manfirstl}}}
# icon bar: widgets
set xfIconBar(bar,widgets) {{button {XFProcConfInsertWidgetDefault Button}} {canvas {XFProcConfInsertWidgetDefault Canvas}} {checkbutton {XFProcConfInsertWidgetDefault Checkbutton}} {entry {XFProcConfInsertWidgetDefault Entry}} {Iconbar-separator {}} {frame {XFProcConfInsertWidgetDefault Frame}} {label {XFProcConfInsertWidgetDefault Label}} {listbox {XFProcConfInsertWidgetDefault Listbox}} {menu {XFProcConfInsertWidgetDefault Menu}} {Iconbar-separator {}} {menubutton {XFProcConfInsertWidgetDefault Menubutton}} {message {XFProcConfInsertWidgetDefault Message}} {radiobutton {XFProcConfInsertWidgetDefault Radiobutton}} {scale {XFProcConfInsertWidgetDefault Scale}} {Iconbar-separator {}} {scrollbar {XFProcConfInsertWidgetDefault Scrollbar}} {text {XFProcConfInsertWidgetDefault Text}} {toplevel {XFProcConfInsertWidgetDefault Toplevel}} {widgetsdel {XFProcIconBarRemove widgets}}}
# eof

