###Title General information on XF
###Keywords XF

If you have problems, please use the help system and read
the FAQ's. If you are new to XF there is an interactive
tutorial available that shows on a small example application
how to work XF. This tutorial is available under
"Help|Tutorial". There is also an complete documentation
available for XF. This is distributed separately, and
contains the help pages for XF. Probably it is not available
yet, so take a look into comp.lang.tcl or the mailing list.

Any comments are welcome. Have fun...

Some special notes on XF:

- There are various bindings, that you can modify. To
  view/change the bindings select the menu item
  "Options|Bindings".

- Before packing/placing you should read the manual page,
  the Tcl book and the help page. It may be a bit
  frustrating to not get the result you want....
