# icon bar configuration
global xfIconBar
# icon bar: edit
set xfIconBar(bar,edit) {{conf {XFProcConfParametersDefault}} {pack {XFProcConfPacking}} {place XFProcConfPlacing} {bind {XFProcConfBinding}} {Iconbar-space {}} {procs XFProcProgProcs} {global XFProcProgGlobals} {tree XFProcProgWidgetTree} {Iconbar-space {}} {cut XFProcEditCut} {copy XFProcEditCopy} {paste XFProcEditPaste} {delete XFProcEditDelete} { Iconbar-space {}} {question {XFProcHelpHelp XF manfirstl}}}
# eof

