###Title General information on the help system
###Keywords Help Tk Tcl XF

If you have problems, please use the help system. Beside the
help page and the manual pages, this help system contains a
list of hints for the current topic, and a note sheet, where
you can write your own notes for the topic (if you have
notes, that seem to be very important and that I have
forgotten, send them to me, so I can include them into help
system (or in other words: please world write my help system
:-)).

Have fun...


