# .frame
# The above line makes pasting MUCH easier for me.
# It contains the pathname of the cutted widget.

# The symbolic names.
global symbolicName

  # build widget .frame
  frame .frame \
    -relief {raised}

  # build widget .frame.scrollbar1
  scrollbar .frame.scrollbar1 \
    -command {.frame.text2 yview} \
    -relief {raised}

  # build widget .frame.scrollbar2
  scrollbar .frame.scrollbar2 \
    -command {.frame.text2 xview} \
    -orient {horizontal} \
    -relief {raised}

  # build widget .frame.text2
  if {[catch "tkemacs .frame.text2 -command TkEmacs -lispfile tkemacs.el"]} {
    if {"[info procs XFEdit]" != ""} {
      XFProcError "Unknown widget type: tkemacs"
      return
    } {
      puts stderr "Unknown widget type: tkemacs"
      destroy .
    }
  }
  .frame.text2 configure \
    -borderwidth {2} \
    -debug {false} \
    -pollinterval {60} \
    -relief {raised} \
    -useadvise {true} \
    -xscrollcommand {.frame.scrollbar2 set} \
    -yscrollcommand {.frame.scrollbar1 set}

  # pack widget .frame
  pack append .frame \
    .frame.scrollbar1   {left frame center filly} \
    .frame.text2   {top frame center expand fill} \
    .frame.scrollbar2   {bottom frame center fillx} 

  # pack widgets
  pack append . \
    .frame   {top fill frame center} 

# end of widget tree

