# .frame
# The above line makes pasting MUCH easier for me.
# It contains the pathname of the cutted widget.

# The symbolic names.
global symbolicName

  # build widget .frame
  frame .frame \
    -relief {raised}

  # build widget .frame.scrollbar1
  scrollbar .frame.scrollbar1 \
    -command {.frame.photo2 yview} \
    -relief {raised}

  # build widget .frame.scrollbar3
  scrollbar .frame.scrollbar3 \
    -command {.frame.photo2 xview} \
    -orient {horizontal} \
    -relief {raised}

  # build widget .frame.photo2
  if {[catch "photo .frame.photo2"]} {
    if {"[info procs XFEdit]" != ""} {
      XFProcError "Unknown widget type: photo"
      return
    } {
      puts stderr "Unknown widget type: photo"
      destroy .
      exit
    }
  }
  .frame.photo2 configure \
    -borderwidth {2} \
    -ditherlevel {0} \
    -palette {7/7/4} \
    -relief {raised} \
    -xscrollcommand {.frame.scrollbar3 set} \
    -yscrollcommand {.frame.scrollbar1 set}

  # pack widget .frame
  pack append .frame \
    .frame.scrollbar1   {left frame center filly} \
    .frame.photo2   {top frame center expand fill} \
    .frame.scrollbar3   {bottom frame center fillx} 

  # pack widgets
  pack append . \
    .frame   {top fill frame center} 

# end of widget tree

